/*
 * @CO2.java Version 1.1 03/08/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.io.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import util.utilFileIO;
import co2.io.co2Read;
import co2.gui.co2StartAppPanel;

/** Class CO2
 *  <p> This Class is the starting point for the CO2 Web App.
 *
 *  @version 1.1 03/08/2012
 *  @author  John R. Victorine
 */

public class CO2 extends Applet
{
  // Runtime object

  private Runtime r = Runtime.getRuntime();

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone = false;

  private String  sKID            = new String("0"); // LAS KID
  private String  sKID_Standalone = new String("0");

  // LAS Viewer Global Variables

  private String     sPath       = new String(".");
  private String     sReadMethod = null;
  private String     sDirectory  = null;
  private String     sFilename   = null;

  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor CO2()
   *  <p> This is the Constructor for this class.
   */

  public CO2() {  }

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Stratigraphic Data Collection and Biostratigraphy Profile
   *      Viewer Module.
   */

  public void init()
  {
    try
    {
      sKID = this.getParameter("sKID", sKID_Standalone);
      if (sKID.equals("undefined"))
        sKID = new String("0");
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    co2StartAppPanel panel = null;

    // Display the LAS Viewer Start

    if (!isStandalone)
    {
      panel = new co2StartAppPanel( null );

      this.setLayout(new BorderLayout());
      this.add(panel, BorderLayout.CENTER);
    }
    else
    {
      JFrame frame = new JFrame();

      frame.setTitle("Enter");
      panel = new co2StartAppPanel( frame );
      frame.getContentPane().add(panel, null);

      // Set the Size and Display the Frame

      frame.setSize(385, 385);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      frame.setLocation((d.width - frame.getSize().width)/2,
                        (d.height - frame.getSize().height)/2);

      frame.setResizable(false);
      frame.setVisible(true);
    }
/*
	int i,j;
	int icnt = 0;
    co2Read pRead      = null;
    int     iTotal     = 0;
    int     iRow       = 0;
    int     iColumns   = 0;
    double  data[][]   = null; // Pressure(psia), Temp (C)
    double  dOut[][]   = null; // Pressure (psia), Temp (C), density (kgs/m^3), viscosity (cp)
	String  sData[][]  = null;
	String  sTemp[]    = null;
	String  sHeader[]  = null;
	String  sColumns[] = null;
	int     iPressure  = 3;
	int     iTemp      = 4;
	int     iDensity   = 5;
	int     iVis       = 6;

    search();

    pRead = new co2Read();
    pRead.setColumnHeaderAt(0);
    pRead.setDataStartAt(1);

    pRead.Open( util.utilFileIO.DIRECTORY, sDirectory, sFilename );
    iTotal = pRead.countLines();
    pRead.Close();

    pRead.Open( util.utilFileIO.DIRECTORY, sDirectory, sFilename );
    pRead.countData();
    pRead.Close();

    pRead.Open( util.utilFileIO.DIRECTORY, sDirectory, sFilename );

    pRead.parseFile();
    iRow     = pRead.getRowCount();
    iColumns = pRead.getColumnCount();
    sData    = pRead.getData();
    sHeader  = pRead.getHeader();
    sTemp    = pRead.getColumns();

    if (sTemp != null)
    {
	  sColumns = new String[sTemp.length+2];

	  for (j=0; j<sTemp.length; j++)
	  {
		sColumns[icnt] = new String( sTemp[j] );
		icnt++;
	  }

	  sColumns[icnt] = new String( "density (kgs/m^3)" );
      icnt++;

	  sColumns[icnt] = new String( "viscosity (cp)" );
      icnt++;
	}

    if (sData != null)
    {
      if (iTotal >0)
      {
        data = new double[iRow][iColumns];
        dOut = new double[iRow][iColumns+2];

        for (i=0; i<iRow; i++)
        {
		  for (j=0; j<iColumns; j++)
		  {
		    data[i][j] = 0.0;
		    if (sData[i][j] != null)
		    {
              if (cmn.cmnString.isNumeric(sData[i][j]))
              {
  	            data[i][j] = cmn.cmnString.stringToDouble(sData[i][j]);
		      }
		    }
		  }

		  for (j=0; j<iColumns+2; j++)
		  {
		    if (j<iColumns)
		      dOut[i][j] = data[i][j];
		    else
		      dOut[i][j] = 0.0;
	  	  }
	    }
      }
	}

    pRead.Close();

    for (i=0; i<iRow; i++)
    {
	  if ( data[i][iPressure] > 0.0)
	  {
	    dOut[i][iDensity] = co2.math.mathCO2.density( data[i][iPressure], data[i][iTemp] );
	    dOut[i][iVis]     = co2.math.mathCO2.viscosity( data[i][iPressure], data[i][iTemp] );
      }
	}

    co2.io.co2Write.write( sDirectory, "Test.csv", sHeader, iRow, iColumns+2, sColumns, dOut );
*/
  }

  /** Method search()
   * <p> This method will search for a Tops File that is on the user's PC
   */

  private void search()
  {
    FileDialog dialog      = null;
    Frame      frameTemp   = new Frame();

    dialog = new FileDialog(frameTemp,
             "Select Comma Delimited File from your PC.");
    dialog.setVisible(true);
    frameTemp.setVisible(true);

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      sReadMethod = new String("FILE");
      sDirectory  = new String(dialog.getDirectory());
      sFilename   = new String(dialog.getFile());
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Biostratigraphy Profile Plot
   */

  public String getAppletInfo() { return ("LAS IO Applet"); }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo = { {"sKID",  "String", "User Primary Key"}, };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    CO2 applet = new CO2();

    applet.isStandalone = true;
    applet.sKID_Standalone = new String("0");
    if (args.length > 0)
      applet.sKID_Standalone = new String(args[0]);

    applet.init();
    applet.start();
  }
}


/*
 *  @version 1.1 03/08/2012
 *  @author  John Victorine
 */
