/*
 * @co2Frame.java Version 1.1 03/30/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package co2.gui;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import co2.io.co2Read;

/** Class co2Frame
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 03/30/2012
 *  @author  John R. Victorine
 */

public class co2Frame extends JFrame implements ActionListener
{
  // Input Variables

  private String sPath     = "";  // Directory Path to CO2 File
  private String sFilename = "";  // Filename of CO2 File

  // Global Variables

  private co2Read pRead    = new co2Read();

  private int     iTotal     = 0;
  private int     iColumns   = -1;
  private int     iUnits     = -1;
  private int     iData      = -1;  // The default data start row i.e. Column Labels are at 0 Row
  private int     iPressure  = -1;
  private int     iTemp      = -1;

  private static final String sRow = "Row 1\nRow 2\nRow 3\nRow 4\nRow 5\n" +
	                                 "Row 6\nRow 7\nRow 8\nRow 9\nRow 10";

  private static final String sQuestions[] = {
	"Is there a File Description Section?",
	"Is there a Row of Column Labels?    ",
	"Is there a Row of Column Units?     ",
  };

  // Global Widgets

  private JTextArea txtRef = new JTextArea();  // 1st 10 Lines
  private JTextArea txtRow = new JTextArea();  // 10 Lines Row

  // File Header Widgets

  private JRadioButton rbHeaderNo   = new JRadioButton();  // No Description Section
  private JRadioButton rbHeaderYes  = new JRadioButton();  // File has Description Section

  private JTextField   txtColumns   = new JTextField();    // Row Number for Column Labels

  private JRadioButton rbColumnsNo  = new JRadioButton();  // No Column Label Row
  private JRadioButton rbColumnsYes = new JRadioButton();  // File has Column Label Row

  private JTextField   txtUnits     = new JTextField();    // Row Number for Column Unit Labels

  private JRadioButton rbUnitsNo    = new JRadioButton();  // No Column Units Label Row
  private JRadioButton rbUnitsYes   = new JRadioButton();  // File has Column Units Label Row

  private JTextField   txtData      = new JTextField();    // Data Row Start Number

  private JTextField   txtPressure  = new JTextField();    // Pressure Column Number
  private JTextField   txtTemp      = new JTextField();    // Temperature Column Number

  private JTextField   txtDirectory = new JTextField();    // Directory Path of output
  private JTextField   txtFilename  = new JTextField();    // Filename of output

  private JButton      btnCompute   = new JButton();       // Compute Button

  /** Constructor co2Frame( Observable pNotifier, iqstratStruct stStruct )
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public co2Frame( String sPath, String sFilename )
  {
    try
    {
      this.sPath     = sPath;
      this.sFilename = sFilename;

      jbInit();

      addWindowListener( new co2Frame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	String      sText         = "";
    JPanel      pnlTop        = new JPanel();
    JPanel      pnlTCenter    = new JPanel();
    JPanel      pnlTEast      = new JPanel();

    JPanel      pnlCenter     = new JPanel();

    JPanel      pnlCNorth     = new JPanel();

    JPanel      pnlRow        = new JPanel();
    JLabel      lblRowCenter  = new JLabel();
    JLabel      lblRow        = new JLabel();

    JPanel      pnlHeader     = new JPanel();
    JLabel      lblHeader     = new JLabel();
    JPanel      pnlCHeader    = new JPanel();
    JPanel      pnlCHeader1   = new JPanel();

    JPanel      pnlColumns    = new JPanel();
    JLabel      lblColumns    = new JLabel();
    JPanel      pnlCColumns   = new JPanel();
    JPanel      pnlCColumns1  = new JPanel();

    JPanel      pnlUnits      = new JPanel();
    JLabel      lblUnits      = new JLabel();
    JPanel      pnlCUnits     = new JPanel();
    JPanel      pnlCUnits1    = new JPanel();

    JPanel      pnlBlank      = new JPanel();

    JPanel      pnlRow1       = new JPanel();
    JLabel      lblRow1Center = new JLabel();
    JLabel      lblRow1       = new JLabel();

    JPanel      pnlData       = new JPanel();
    JLabel      lblData       = new JLabel();

    JPanel      pnlBlank2      = new JPanel();

    JPanel      pnlRow2       = new JPanel();
    JLabel      lblRow2Center = new JLabel();
    JLabel      lblRow2       = new JLabel();

    JPanel      pnlPressure   = new JPanel();
    JLabel      lblPressure   = new JLabel();

    JPanel      pnlTemp       = new JPanel();
    JLabel      lblTemp       = new JLabel();

    JPanel      pnlBase       = new JPanel();

    JPanel      pnlOutput     = new JPanel();
    JPanel      pnlDirectory  = new JPanel();
    JPanel      pnlFilename   = new JPanel();
    JPanel      pnlCompute    = new JPanel();

    JScrollPane scroll        = new JScrollPane();

    ButtonGroup groupHeader   = new ButtonGroup();
    ButtonGroup groupColumns  = new ButtonGroup();
    ButtonGroup groupUnits    = new ButtonGroup();

    TitledBorder titledBorder1st10 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "1st 10 Lines of File:");
    titledBorder1st10.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDirectory = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Directory:");
    titledBorderDirectory.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderFilename = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Output Filename:");
    titledBorderFilename.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Get 1st 10 Lines

    pRead = new co2Read();

    pRead.Open( util.utilFileIO.DIRECTORY, sPath, sFilename );
    iTotal = pRead.countLines();
    pRead.Close();

    pRead.Open( util.utilFileIO.DIRECTORY, sPath, sFilename );
    sText = pRead.parse10Lines();
    pRead.Close();

    // Build Other Widgets

    this.setTitle("Predicting Density and Viscosity or Supercritical CO2");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());
    pnlTop.setBorder( titledBorder1st10 );

    // -- 1st 10 Lines of File Panel

    pnlTCenter.setLayout(new BorderLayout());
    pnlTCenter.setBorder(BorderFactory.createEtchedBorder());

    txtRef.setText(sText);
    txtRef.setCaretPosition(0);
    txtRef.setEditable(false);
    txtRef.setRows(10);
    txtRef.setLineWrap(false);
    txtRef.setWrapStyleWord(false);
    txtRef.addFocusListener(new co2FrameFocusAdapter(this));

    // -- Row numbers Panel

    pnlTEast.setLayout(new BorderLayout());
    pnlTEast.setBorder(BorderFactory.createEtchedBorder());

    txtRow.setText(sRow);
    txtRow.setCaretPosition(0);
    txtRow.setEditable(false);
    txtRow.setBackground( new Color(240, 240, 240) );
    txtRow.setColumns(6);
    txtRow.setRows(10);
    txtRow.setLineWrap(false);
    txtRow.setWrapStyleWord(false);
    txtRow.addFocusListener(new co2FrameFocusAdapter(this));

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Header Section of File Panel

    pnlCNorth.setLayout(new GridLayout(11,1));

    // .. Row Number Panel

    pnlRow.setLayout(new BorderLayout());

    lblRowCenter.setText(".         File Header Section");
    lblRow.setText( "Row Number (1,2,3...)" );

    // .. File Description Well, Field Area Panel

    pnlHeader.setLayout(new BorderLayout());

    lblHeader.setText( sQuestions[0] );
    lblHeader.setFont(new java.awt.Font("Monospaced", 1, 12));

    pnlCHeader.setLayout(new BorderLayout());

    rbHeaderNo.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbHeaderNo.setText( "No" );
	rbHeaderNo.setSelected(true);
	rbHeaderNo.addActionListener(this);

    groupHeader.add(rbHeaderNo);

    pnlCHeader1.setLayout(new BorderLayout());

    rbHeaderYes.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbHeaderYes.setText( "Yes" );
	rbHeaderYes.addActionListener(this);

    groupHeader.add(rbHeaderYes);

    // .. Column Names of File Data Panel

    pnlColumns.setLayout(new BorderLayout());

    lblColumns.setText( sQuestions[1] );
    lblColumns.setFont(new java.awt.Font("Monospaced", 1, 12));

    pnlCColumns.setLayout(new BorderLayout());

    rbColumnsNo.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbColumnsNo.setText( "No" );
	rbColumnsNo.setSelected(true);
	rbColumnsNo.addActionListener(this);

    groupColumns.add(rbColumnsNo);

    pnlCColumns1.setLayout(new BorderLayout());

    rbColumnsYes.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbColumnsYes.setText( "Yes" );
	rbColumnsYes.addActionListener(this);

    groupColumns.add(rbColumnsYes);

    txtColumns.setText( "-1" );
    txtColumns.setColumns( 6 );
    txtColumns.setHorizontalAlignment(SwingConstants.TRAILING);
    txtColumns.addFocusListener(new co2FrameFocusAdapter(this));

    // .. Column Units of File Data Panel

    pnlUnits.setLayout(new BorderLayout());

    lblUnits.setText( sQuestions[2] );
    lblUnits.setFont(new java.awt.Font("Monospaced", 1, 12));

    pnlCUnits.setLayout(new BorderLayout());

    rbUnitsNo.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbUnitsNo.setText( "No" );
	rbUnitsNo.setSelected(true);
	rbUnitsNo.addActionListener(this);

    groupUnits.add(rbUnitsNo);

    pnlCUnits1.setLayout(new BorderLayout());

    rbUnitsYes.setFont(new java.awt.Font("Monospaced", 1, 12));
	rbUnitsYes.setText( "Yes" );
	rbUnitsYes.addActionListener(this);

    groupUnits.add(rbUnitsYes);

    txtUnits.setText( "-1" );
    txtUnits.setColumns( 6 );
    txtUnits.setHorizontalAlignment(SwingConstants.TRAILING);
    txtUnits.addFocusListener(new co2FrameFocusAdapter(this));

    // .. Data Row Number Panel

    pnlRow1.setLayout(new BorderLayout());

    lblRow1Center.setText(".         Data Section");
    lblRow1.setText( "Row Number (1,2,3...)" );

    // .. Data Row Number Panel

    pnlData.setLayout(new BorderLayout());

    lblData.setText("Start Reading Data at Row (1,2,3...)");

    txtData.setText( "-1" );
    txtData.setColumns( 6 );
    txtData.setHorizontalAlignment(SwingConstants.TRAILING);
    txtData.addFocusListener(new co2FrameFocusAdapter(this));

    // .. Data Column Number Panel

    pnlRow2.setLayout(new BorderLayout());

    lblRow2Center.setText(".         Pressure & Temperature Column Numbers");
    lblRow2.setText( "Column (1,2,3...)" );

    // .. Pressure Column Number Panel

    pnlPressure.setLayout(new BorderLayout());

    lblPressure.setText( "Pressure Column Number is?" );
    lblPressure.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtPressure.setText( "-1" );
    txtPressure.setColumns( 6 );
    txtPressure.setHorizontalAlignment(SwingConstants.TRAILING);
    txtPressure.addFocusListener(new co2FrameFocusAdapter(this));

    // .. Temperature Column Number Panel

    pnlTemp.setLayout(new BorderLayout());

    lblTemp.setText( "Temperature Column Number is?" );
    lblTemp.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtTemp.setText( "-1" );
    txtTemp.setColumns( 6 );
    txtTemp.setHorizontalAlignment(SwingConstants.TRAILING);
    txtTemp.addFocusListener(new co2FrameFocusAdapter(this));

    // Base Panel

    pnlBase.setLayout(new BorderLayout());

    pnlOutput.setLayout(new GridLayout(2,1));

    pnlDirectory.setLayout(new BorderLayout());
    pnlDirectory.setBorder( titledBorderDirectory );

    txtDirectory.setText( sPath );
    txtDirectory.setEditable( false );

    pnlFilename.setLayout(new BorderLayout());
    pnlFilename.setBorder( titledBorderFilename );

    txtFilename.setText( "out_" + sFilename );

    // -- Compute Button Panel

    pnlCompute.setLayout(new BorderLayout());

    btnCompute.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCompute.setText("Compute Density & Viscosity");
    btnCompute.addActionListener(this);

    // Set Widgets depending on Data

    setWidgets();

    // Add Widgets to Frame

    this.getContentPane().add( pnlTop,    BorderLayout.NORTH);
    pnlTop.add(pnlTCenter,                BorderLayout.CENTER);
    pnlTCenter.add(txtRef,                BorderLayout.CENTER);

    pnlTop.add(pnlTEast,                  BorderLayout.WEST);
    pnlTEast.add(txtRow,                  BorderLayout.CENTER);

    this.getContentPane().add( pnlCenter, BorderLayout.CENTER);
    pnlCenter.add(pnlCNorth,              BorderLayout.NORTH);

    pnlCNorth.add(pnlRow,                 null);
    pnlRow.add(lblRowCenter,              BorderLayout.CENTER);
    pnlRow.add(lblRow,                    BorderLayout.EAST);

    pnlCNorth.add(pnlHeader,              null);
    pnlHeader.add(lblHeader,              BorderLayout.WEST);

    pnlHeader.add(pnlCHeader,             BorderLayout.CENTER);
    pnlCHeader.add(rbHeaderNo,            BorderLayout.WEST);

    pnlCHeader.add(pnlCHeader1,           BorderLayout.CENTER);
    pnlCHeader1.add(rbHeaderYes,          BorderLayout.WEST);

    pnlCNorth.add(pnlColumns,             null);
    pnlColumns.add(lblColumns,            BorderLayout.WEST);

    pnlColumns.add(pnlCColumns,           BorderLayout.CENTER);
    pnlCColumns.add(rbColumnsNo,          BorderLayout.WEST);

    pnlCColumns.add(txtColumns,           BorderLayout.EAST);

    pnlCColumns.add(pnlCColumns1,         BorderLayout.CENTER);
    pnlCColumns1.add(rbColumnsYes,        BorderLayout.WEST);

    pnlCNorth.add(pnlUnits,               null);
    pnlUnits.add(lblUnits,                BorderLayout.WEST);

    pnlUnits.add(pnlCUnits,               BorderLayout.CENTER);
    pnlCUnits.add(rbUnitsNo,              BorderLayout.WEST);

    pnlCUnits.add(txtUnits,               BorderLayout.EAST);

    pnlCUnits.add(pnlCUnits1,             BorderLayout.CENTER);
    pnlCUnits1.add(rbUnitsYes,            BorderLayout.WEST);

    pnlCNorth.add(pnlBlank,               null);

    pnlCNorth.add(pnlRow1,                null);
    pnlRow1.add(lblRow1Center,            BorderLayout.CENTER);
//    pnlRow1.add(lblRow1,                  BorderLayout.EAST);

    pnlCNorth.add(pnlData,                null);
    pnlData.add(lblData,                  BorderLayout.WEST);
    pnlData.add(txtData,                  BorderLayout.EAST);

    pnlCNorth.add(pnlBlank2,              null);

    pnlCNorth.add(pnlRow2,                null);
    pnlRow2.add(lblRow2Center,            BorderLayout.CENTER);
    pnlRow2.add(lblRow2,                  BorderLayout.EAST);

    pnlCNorth.add(pnlPressure,            null);
    pnlPressure.add(lblPressure,          BorderLayout.WEST);
    pnlPressure.add(txtPressure,          BorderLayout.EAST);

    pnlCNorth.add(pnlTemp,                null);
    pnlTemp.add(lblTemp,                  BorderLayout.WEST);
    pnlTemp.add(txtTemp,                  BorderLayout.EAST);

    this.getContentPane().add( pnlBase,   BorderLayout.SOUTH);

    pnlBase.add(pnlOutput,                BorderLayout.CENTER);
    pnlOutput.add(pnlDirectory,           null);
    pnlDirectory.add(txtDirectory,        BorderLayout.CENTER);

    pnlOutput.add(pnlFilename,            null);
    pnlFilename.add(txtFilename,          BorderLayout.CENTER);

    pnlBase.add(pnlCompute,               BorderLayout.SOUTH);
    pnlCompute.add(btnCompute,            BorderLayout.SOUTH);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(new Dimension(550, 650));
    this.setLocation( (d.width - this.getSize().width)/2,
                      (d.height - this.getSize().height)/2);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD co2Frame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class co2Frame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    String sPath     = null;  // Directory Path to CO2 File
    String sFilename = null;  // Filename of CO2 File

    if (pRead != null)
      pRead.delete();
    pRead = null;

    // Global Widgets

    txtRef       = null; // 1st 10 Lines
    txtRow       = null; // 10 Lines Row

    // File Header Widgets

    rbHeaderNo   = null; // No Description Section
    rbHeaderYes  = null; // File has Description Section

    txtColumns   = null; // Row Number for Column Labels

    rbColumnsNo  = null; // No Column Label Row
    rbColumnsYes = null; // File has Column Label Row

    txtUnits     = null; // Row Number for Column Unit Labels

    rbUnitsNo    = null; // No Column Units Label Row
    rbUnitsYes   = null; // File has Column Units Label Row

    txtData      = null; // Data Row Start Number

    txtPressure  = null; // Pressure Column Number
    txtTemp      = null; // Temperature Column Number

    txtDirectory = null; // Directory Path of output
    txtFilename  = null; // Filename of output

    btnCompute   = null; // Compute Button

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- SET ACTIONS ------------------------------ *
   * ===================================================================== */

  /** Method setWidgets()
   *  <p> This method will set the Widgets depending on data entered.
   */

  private void setWidgets()
  {
	int iTemp = 0;

    if (txtColumns != null)
    {
      txtColumns.setEditable( false );  // Row Number for Column Labels
      txtUnits.setEditable( false );    // Row Number for Column Unit Labels
      btnCompute.setEnabled( false );   // Compute Button

      txtColumns.setText( "-1" );
      if (rbColumnsYes.isSelected())    // File has Description Section
      {
        txtColumns.setEditable( true ); // Row Number for Column Labels
        iTemp = iColumns + 1;
        txtColumns.setText( "" + iTemp );
	  }

      txtUnits.setText( "-1" );
      if (rbUnitsYes.isSelected())      // File has Description Section
      {
        txtUnits.setEditable( true );   // Row Number for Column Labels
        iTemp = iUnits + 1;
        txtUnits.setText( "" + iTemp );
	  }

	  if ((iData > -1) && (iPressure > -1) && (iTemp > -1))
	  {
        btnCompute.setEnabled( true );  // Compute Button
      }
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method compute()
   * <p> This method will read, parse and compute the density and viscosity
   *     and then write the two new columns to a new file.
   */

  public void compute()
  {
	int    i,j;
	int    icnt       = 0;
	int    iRow       = 0;
	int    iColumn    = 0;
	String sOut[][]   = null;
	String sData[][]  = null;
	String sTemp[]    = null;
	String sHeader[]  = null;
	String sColumns[] = null;
	String sUnits[]   = null;
	double dPressure  = 0.0;
	double dTemp      = 0.0;
    int    iDensity   = -1;
    int    iViscosity = -1;

    pRead.setColumnHeaderAt( iColumns );
	pRead.setUnitHeaderAt( iUnits );
    pRead.setDataStartAt( iData );

    pRead.Open( util.utilFileIO.DIRECTORY, sPath, sFilename );
    pRead.countData();
    pRead.Close();

    pRead.Open( util.utilFileIO.DIRECTORY, sPath, sFilename );

    pRead.parseFile();
    iRow       = pRead.getRowCount();
    iColumn    = pRead.getColumnCount();
    iDensity   = iColumn;
    iViscosity = iDensity + 1;

    sData      = pRead.getData();
    sHeader    = pRead.getHeader();

    icnt  = 0;
    sTemp = pRead.getColumns();

    if (sTemp != null)
    {
	  sColumns = new String[sTemp.length+2];

	  for (j=0; j<sTemp.length; j++)
	  {
		sColumns[icnt] = new String( sTemp[j] );
		icnt++;
	  }

      if (iUnits > -1)
      {
	    sColumns[icnt] = new String( "density" );
        icnt++;

	    sColumns[icnt] = new String( "viscosity " );
        icnt++;
      }
      else
      {
	    sColumns[icnt] = new String( "density (kgs/m^3)" );
        icnt++;

	    sColumns[icnt] = new String( "viscosity (cp)" );
        icnt++;
	  }
	}

    icnt  = 0;
    sTemp = pRead.getUnits();

    if (sTemp != null)
    {
	  sUnits = new String[sTemp.length+2];

	  for (j=0; j<sTemp.length; j++)
	  {
		sUnits[icnt] = new String( sTemp[j] );
		icnt++;
	  }

	  sUnits[icnt] = new String( "kgs/m^3" );
      icnt++;

	  sUnits[icnt] = new String( "cp" );
      icnt++;
	}

    if (sData != null)
    {
      if (iTotal >0)
      {
        sOut = new String[iRow][iColumn+2];

        for (i=0; i<iRow; i++)
        {
		  for (j=0; j<iColumn+2; j++)
		  {
		    if (j<iColumn)
		      sOut[i][j] = new String( "" + sData[i][j] );
		    else
		      sOut[i][j] = "0.0";
	  	  }
	    }
      }
	}

    pRead.Close();

    for (i=0; i<iRow; i++)
    {
	  dPressure = 0.0;
	  if (cmn.cmnString.isNumeric(sData[i][iPressure]))
	  {
		dPressure = cmn.cmnString.stringToDouble(sData[i][iPressure]);
	  }

	  dTemp = 0.0;
	  if (cmn.cmnString.isNumeric(sData[i][iTemp]))
	  {
		dTemp = cmn.cmnString.stringToDouble(sData[i][iTemp]);
	  }

      if (dPressure > 0.0)
      {
        sOut[i][iDensity]   = new String( "" +
                              co2.math.mathCO2.density( dPressure, dTemp ));
        sOut[i][iViscosity] = new String( "" +
                              co2.math.mathCO2.viscosity( dPressure, dTemp ));
      }
	}

    co2.io.co2Write.write( sPath, txtFilename.getText(),
                           sHeader, iRow, iColumn+2, sColumns, sUnits, sOut );

    close();
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	if (event.getSource() == btnCompute) { compute(); }

	setWidgets();
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric  = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (e.getSource() == txtColumns)
    {
	  iNumeric = 1;
	  sTemp    = txtColumns.getText();
	  sMessage = new String( "Data Column Row Value is a Numeric Field");
	}

    if (e.getSource() == txtUnits)
    {
	  iNumeric = 1;
	  sTemp    = txtUnits.getText();
	  sMessage = new String( "Data Unit Row Value is a Numeric Field");
	}

    if (e.getSource() == txtData)
    {
	  iNumeric = 1;
	  sTemp    = txtData.getText();
	  sMessage = new String( "Data Starting Row Value is a Numeric Field");
	}

    if (e.getSource() == txtPressure)
    {
	  iNumeric = 1;
	  sTemp    = txtPressure.getText();
	  sMessage = new String( "Pressure Column Value is a Numeric Field");
	}

    if (e.getSource() == txtTemp)
    {
	  iNumeric = 1;
	  sTemp    = txtTemp.getText();
	  sMessage = new String( "Temperature Column Value is a Numeric Field");
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog( (Component) null,
                                        sMessage,
                                        "ERROR",
                                        JOptionPane.ERROR_MESSAGE );
      }
      else
      {
        if (e.getSource() == txtColumns)
        {
		  iColumns = (int) cmn.cmnString.stringToDouble(sTemp) - 1;
  	    }

        if (e.getSource() == txtUnits)
        {
		  iUnits = (int) cmn.cmnString.stringToDouble(sTemp) - 1;
	    }

        if (e.getSource() == txtData)
        {
		  iData = (int) cmn.cmnString.stringToDouble(sTemp) - 1;
	    }

        if (e.getSource() == txtPressure)
        {
		  iPressure = (int) cmn.cmnString.stringToDouble(sTemp) - 1;
	    }

        if (e.getSource() == txtTemp)
        {
	      iTemp = (int) cmn.cmnString.stringToDouble(sTemp) - 1;
	    }
  	  }
    }

	setWidgets();
  }
}

/** CLASS co2FrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class co2FrameFocusAdapter extends java.awt.event.FocusAdapter
{
  co2Frame adaptee;

  co2FrameFocusAdapter(co2Frame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 03/30/2012
 *  @author  John Victorine
 */
