/*
 * @co2StartAppPanel.java Version 1.1 03/30/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package co2.gui;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import co2.gui.co2Frame;          // CO2 Control Data Dialog

/** Class co2StartAppPanel
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 03/30/2012
 *  @author  John R. Victorine
 */

public class co2StartAppPanel extends JPanel implements ActionListener
{
  // Input Variables

  private JFrame   parent   = null;  // Frame calling this panel

  // Global Variable

  private co2Frame frame       = null;

  private String   sReadMethod = null;
  private String   sDirectory  = null;
  private String   sFilename   = null;

  // Global Panel Widgets

  private JButton    btn      = null;
  private JButton    btnExit  = new JButton();

  /** Constructor co2StartAppPanel( JFrame parent )
   *  <p> This is the Constructor for this class.
   *  @param parent   = the frame calling this panel
   */

  public co2StartAppPanel( JFrame parent )
  {
    try
    {
      this.parent   = parent;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlCenter = new JPanel();
    JPanel pnlBase   = new JPanel();

    JLabel lbl       = new JLabel();

    // Build Panel

    this.setLayout( new BorderLayout() );
    this.setBackground(Color.white);

    pnlCenter.setLayout( new BorderLayout() );
    pnlCenter.setBackground(Color.white);

    btn = new JButton( createImageIcon( "/images/co2.png" ) );
    btn.setBackground(Color.white);
    btn.addActionListener( this );

    lbl.setHorizontalAlignment(SwingConstants.CENTER);
    lbl.setFont(new java.awt.Font("Dialog", 1, 20));
    lbl.setBackground(Color.white);
    lbl.setText("Enter");

    pnlBase.setLayout( new GridLayout() );
    pnlBase.setBackground(Color.white);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.add( pnlCenter,    BorderLayout.CENTER );
    pnlCenter.add( btn,     BorderLayout.CENTER );
    pnlCenter.add( lbl,     BorderLayout.SOUTH );

    if (parent != null)
    {
      this.add( pnlBase,    BorderLayout.SOUTH );
      pnlBase.add( btnExit, null );
    }
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /** Method search()
   * <p> This method will search for a Tops File that is on the user's PC
   */

  private void search()
  {
    FileDialog dialog      = null;
    Frame      frameTemp   = new Frame();

    dialog = new FileDialog(frameTemp,
             "Select Comma Delimited File from your PC.");
    dialog.setVisible(true);
    frameTemp.setVisible(true);

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      sReadMethod = new String("FILE");
      sDirectory  = new String(dialog.getDirectory());
      sFilename   = new String(dialog.getFile());
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    if (frame != null)
      frame.close();
    frame    = null;

    sReadMethod = null;
    sDirectory  = null;
    sFilename   = null;

    // Global Panel Widgets

    btn      = null;
    btnExit  = null;

    if (parent != null)
    {
      parent.dispose();
      parent = null;

      System.exit(0);
    }
  }

  /** Method callCO2Frame()
   * <p> This method will open the CO2 Frame to get CO2 File
   */

  private void callCO2Frame()
  {
	search();

    if (frame != null)
      frame.close();
    frame    = null;

    if ((sDirectory != null) && (sFilename != null))
    {
      frame = new co2Frame( sDirectory, sFilename );
    }
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btn)     { callCO2Frame(); }
    if (event.getSource() == btnExit) { close(); }
  }
}

/*
 *  @version 1.1 03/30/2012
 *  @author  John Victorine
 */
