/*
 * @co2Read.java Version 1.1 03/30/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package co2.io;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import util.utilFileIO;
import cmn.cmnString;

/** Class co2Read
 *  <p> This Class is designed to open an ascii text file and read
 *      the contents and parse the data into the regions Structure.
 *
 *  @version 1.1 03/30/2012
 *  @author  John R. Victorine
 */

public class co2Read
{
  private utilFileIO pFILE       = null;  // File Read Class

  private int        iLines      = 0;     // Total Number of Lines in File
  private int        iRows       = 0;     // Total Number of Rows
  private int        iColumns    = 0;     // Total Number of Columns
  private String     sFile       = "";    // 1st 10 lines of File
  private String     sHeader[]   = null;  // Header information lines
  private String     sLabels[]   = null;  // Column Labels for data
  private String     sUnits[]    = null;  // Column Unit Labels for data
  private String     sData[][]   = null;  // Data Array
  private int        iColHeader  = -1;    // Column Header Row Number
  private int        iUnitHeader = -1;    // Unit Header Row Number
  private int        iStartAt    = 0;     // starting line number of retrieving data

  private int        iError      = util.utilFileIO.NOERROR;
  private String     sError      = new String("");

  /** Construct co2Read()
   *  <p> This is the Constructor for this class.
   */

  public co2Read() { }

  /** Method delete()
   *  <p> This method will deallocate the memory used by this class
   */

  public void delete()
  {
	if (pFILE != null)
	{
	  pFILE.delete();
	  pFILE = null;
	}

    sFile   = null;  // 1st 10 lines of File
    sHeader = null;  // Header information lines
    sLabels = null;  // Column Labels for data
    sUnits  = null;  // Column Unit Labels for data
    sData   = null;  // Data Array
    sError  = null;
  }

  /** Method Open()
    * <p> This is the Constructor for this class.
    * @param iFileID    = To access the file as URL or FILE.
    * @param sDirectory = File URL or Directory Path
    * @param sFilename  = File Name
    */

  public void Open(int iFileID, String sDirectory, String sFilename)
  {
    pFILE  = new utilFileIO();
    pFILE.Open(iFileID, util.utilFileIO.READ, sDirectory, sFilename);

    iError = pFILE.getErrorID();
    sError = pFILE.getError();
  }

  /** Method Close()
   *  <p> This method closes the File.
   */

  public void Close() { pFILE.Close(); }

  /** Method countLines()
   *  <p> This method will count the number of lines in the file
   *  @return iLines = the total number of lines
   */

  public int countLines()
  {
    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
		  iLines++;
		}
	  }
    }

    return (iLines);
  }

  /** Method countData()
   *  <p> This method will count the number of data rows in the file
   *  @return iLines = the total number of lines
   */

  public int countData()
  {
	int    iCount = 0;
	String sTemp  = "";

    iRows = 0;

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
		  sTemp = pFILE.getString();

          if (iCount >= iStartAt)
          {
            iRows++;
   	      }

		  iCount++;
		}
	  }
    }

    return (iRows);
  }

  /** Method parse10Lines()
   * <p> This method will read and parse the 1st 10 lines of Files.
   * @param sFile = 1st 10 Lines of the file
   */

  public String parse10Lines()
  {
	int    iCount = 0;
	int    iRow   = 0;
	String sTemp  = "";
    String sFile  = "";

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
		  if (iCount < 10)
		  {
            sTemp = pFILE.getString();
            if (iCount == 0)
              sFile = new String( sTemp + "\n");
            else
              sFile = new String( sFile + " " + sTemp  + "\n" );
		  }

		  iCount++;
		}
	  }
	}

	return (sFile);
  }

  /** Method parseFile()
   * <p> This method will read and parse the File.
   */

  public void parseFile()
  {
	int    iCount = 0;
	int    iRow   = 0;
	String sTemp  = "";
    String sRow[] = null;

    if (iError == util.utilFileIO.NOERROR)
    {
      while (!pFILE.Next().equals(util.utilFileIO.EOF))
      {
        iError = pFILE.getErrorID();
        sError = pFILE.getError();

        if (iError == util.utilFileIO.NOERROR)
        {
		  if (iCount < iStartAt)
		  {
            sTemp = pFILE.getString();

            // Is this row a list of Column Names?
            if (iCount == iColHeader)
            {
			  sLabels = parseRow( sTemp );
			}
            // Is this row a list of Units?
            else if (iCount == iUnitHeader)
            {
			  sUnits = parseRow( sTemp );
			}
			// Otherwise
			else
			{
			  sHeader = addString( sTemp, sHeader );
			}
		  }

          if (iCount >= iStartAt)
          {
            sTemp = pFILE.getString();

            sRow  = parseRow( sTemp );

			if (iCount == iStartAt)
			{
			  if (sRow.length > 1)
			  {
			    iColumns = sRow.length;
			    sData    = new String[iRows][iColumns];
		      }
			}

            for (int j=0; j<iColumns; j++)
            {
			  if (sRow.length == iColumns)
			  {
			    if (iRow < iRows)
			    sData[iRow][j] = new String(  sRow[j] );
			  }
			}

            iRow++;
	      }

          iCount++;
		}
	  }
    }
  }

  /** Method parseRow()
   * <p> This method will parse a Tops File row into the regions structure.
   * @param  sLine  = Data Row String
   */

  private String[] parseRow( String sLine )
  {
	int             iColumns = 0;
    int             iCount   = 0;
    int             iFound   = 0;
    int             iNext    = 0;
    String          sTemp    = "";
    String          sRow[]   = null;
    StringTokenizer stToken  = null;

	if (sLine != null)
	{
      sTemp    = new String(sLine.replace('\t', ' '));
      stToken  = new StringTokenizer(sTemp, ",");
      iColumns = stToken.countTokens();

      if (iColumns > 1)
      {
        sRow  = new String[iColumns];

        for (int i=0; i<iColumns; i++)
        {
          sTemp = stToken.nextToken();
          sTemp = new String(sTemp.trim());

          sRow[i] = new String( sTemp );
        }
      }
	}

	return (sRow);
  }

  /* ====================================================================== *
   * ----------------------------- GET METHODS ---------------------------- *
   * ====================================================================== */

  /** Method getTotalLines()
   *  <p> This method will return the total number of lines in the CSV File
   *  @return iLines = the total number of lines in the CSV File.
   */

  public int getTotalLines() { return (iLines); }

  /** Method getRowCount()
   *  <p> This method will return the total number of data rows
   *  @return iRows = the total number of data rows.
   */

  public int getRowCount() { return (iRows); }

  /** Method getColumns()
   *  <p> This method will return the total number of data columns
   *  @return iColumns = the total number of data columns.
   */

  public int getColumnCount() { return (iColumns); }

  /** Method getHeader()
   *  <p> This method will return the upper list of strings that are not part
   *      of the data section.
   *  @return sHeader = the upper list of strings that are not part of the data section.
   */

  public String[] getHeader() { return (sHeader); }

  /** Method getColumns()
   *  <p> This method will return the columns to the data set
   *  @return sLabels = the data column Labels.
   */

  public String[] getColumns() { return (sLabels); }

  /** Method getUnits()
   *  <p> This method will return the units to the data set
   *  @return sUnits = the data column Unit Labels.
   */

  public String[] getUnits() { return (sUnits); }

  /** Method getData()
   *  <p> This method will return the data array
   *  @return sUnits = the data array.
   */

  public String[][] getData() { return (sData); }

  /* ====================================================================== *
   * ----------------------------- SET METHODS ---------------------------- *
   * ====================================================================== */

  /** Method setColumnHeaderAt()
   *  <p> This method will set the line number of Column Header
   *  @return iColHeader = starting line number of the Column Header
   */

  public void setColumnHeaderAt( int iColHeader ) { this.iColHeader = iColHeader; }

  /** Method setDataStartAt()
   *  <p> This method will set the starting line number of Column Unit Header
   *  @return iUnitHeader = starting line number of the Column Unit Header
   */

  public void setUnitHeaderAt( int iUnitHeader ) { this.iUnitHeader = iUnitHeader; }

  /** Method setDataStartAt()
   *  <p> This method will set the starting line number of retrieving data
   *  @return iStartAt = starting line number of retrieving data
   */

  public void setDataStartAt( int iStartAt ) { this.iStartAt = iStartAt; }

  /* ====================================================================== *
   * ----------------------------- ADD METHODS ---------------------------- *
   * ====================================================================== */

  /** Method addString()
   * <p> This method will add a string to an existing list of strings
   * @param  str    = new string
   * @param  sArray = existing list of strings
   * @return sArray = the list with the new string added.
   */

  public static String[] addString( String str, String sArray[] )
  {
    int    i,j;
    int    iRecords = 0;
    int    iCount   = 0;
    String sTemp[]  = null;

    if (sArray != null)
      iRecords = sArray.length+1;
    else
      iRecords = 1;

    sTemp = new String[iRecords];

    if (sArray != null)
    {
      for (i=0; i<sArray.length; i++)
      {
        if (iCount < iRecords)
        {
          sTemp[iCount] = new String( sArray[i] );
          iCount++;
        }
      }

      sArray = null;
    }

    sTemp[iCount] = new String( str );
    iCount++;

    sArray = new String[iCount];

    for (i=0; i<iCount; i++)
    {
      sArray[i] = new String( sTemp[i] );
    }

    sTemp = null;

    return (sArray);
  }

  /* ====================================================================== *
   * -------------------------- GET ERROR METHODS ------------------------- *
   * ====================================================================== */

  /** METHOD getErrorID()
   *  <p> This method will return the error number.
   *  @return iError = Error Indicator
   */

  public int getErrorID() { return (iError); }

  /** METHOD getError()
   *  <p> This method will return the error string.
   *  @return sError = Error String
   */

  public String getError() { return (sError); }
}

/*
 *  @version 1.1 03/30/2012
 *  @author  John Victorine
 */
