/*
 * @co2Write.java Version 1.1 03/30/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package co2.io;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import util.utilFileIO;
import cmn.cmnString;

/** Class co2Write
 *  <p> This Class is designed to open an ascii text file and read
 *      the contents and parse the data into the regions Structure.
 *
 *  @version 1.1 03/30/2012
 *  @author  John R. Victorine
 */

public class co2Write
{
  /** Method write()
   *  <p> This method will write the density & viscosity to a CSV File with the original data.
   *  @param sPath = The directory path of the output file
   *  @param sFilename = the filename of the output file
   *  @param iRows     = The total number of data rows
   *  @param iColumns  = The total number of data columns
   *  @param sColumn   = The Column header
   *  @param dOut      = The data array
   */

  public static void write( String sPath,
                            String sFilename,
                            String sHeader[],
                            int    iRows,
                            int    iColumns,
                            String sColumn[],
                            double dOut[][] )
  {
	int i,j;
    String     sOut   = "";
    utilFileIO pIO    = new utilFileIO();
    int        iError = util.utilFileIO.NOERROR;
    String     sError = "";

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    iError = pIO.getErrorID();
    sError = pIO.getError();

    // Write the Column Headers to the data set

    if (sHeader != null)
    {
	  for (i=0; i<sHeader.length; i++)
	  {
        sOut = new String( sHeader[i] + "\r\n");
        pIO.Write(sOut);
      }
	}

    if (sColumn != null)
    {
      for (j=0; j<iColumns; j++)
      {
        if (j==0)
          sOut = new String( sColumn[j] );
        else
          sOut = new String( sOut + "," + sColumn[j] );
      }
      sOut = new String( sOut + "\r\n");
      pIO.Write(sOut);
    }

    // Write the data array

    for (i=0; i<iRows; i++)
    {
	  sOut = new String( "" );
      for (j=0; j<iColumns; j++)
      {
        if (j==0)
          sOut = new String( "" + dOut[i][j] );
        else
          sOut = new String( sOut + "," + dOut[i][j] );
      }
      sOut = new String( sOut + "\r\n");

      pIO.Write(sOut);
    }


    // Close the File

    if (iError == util.utilFileIO.NOERROR)
      pIO.Close();
  }

  /** Method write()
   *  <p> This method will write the density & viscosity to a CSV File with the original data.
   *  @param sPath = The directory path of the output file
   *  @param sFilename = the filename of the output file
   *  @param iRows     = The total number of data rows
   *  @param iColumns  = The total number of data columns
   *  @param sColumn   = The Column header
   *  @param sUnits    = The Column header
   *  @param sData     = The data array
   */

  public static void write( String sPath,
                            String sFilename,
                            String sHeader[],
                            int    iRows,
                            int    iColumns,
                            String sColumn[],
                            String sUnits[],
                            String sData[][] )
  {
	int i,j;
    String     sOut   = "";
    utilFileIO pIO    = new utilFileIO();
    int        iError = util.utilFileIO.NOERROR;
    String     sError = "";

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    iError = pIO.getErrorID();
    sError = pIO.getError();

    // Write the Column Headers to the data set

    if (sHeader != null)
    {
	  for (i=0; i<sHeader.length; i++)
	  {
        sOut = new String( sHeader[i] + "\r\n");
        pIO.Write(sOut);
      }
	}

    if (sColumn != null)
    {
      for (j=0; j<iColumns; j++)
      {
        if (j==0)
          sOut = new String( sColumn[j] );
        else
          sOut = new String( sOut + "," + sColumn[j] );
      }
      sOut = new String( sOut + "\r\n");
      pIO.Write(sOut);
    }

    if (sUnits != null)
    {
      for (j=0; j<sUnits.length; j++)
      {
        if (j==0)
          sOut = new String( sUnits[j] );
        else
          sOut = new String( sOut + "," + sUnits[j] );
      }
      sOut = new String( sOut + "\r\n");
      pIO.Write(sOut);
    }

    // Write the data array

    for (i=0; i<iRows; i++)
    {
	  sOut = new String( "" );
      for (j=0; j<iColumns; j++)
      {
        if (j==0)
          sOut = new String( sData[i][j] );
        else
          sOut = new String( sOut + "," + sData[i][j] );
      }
      sOut = new String( sOut + "\r\n");

      pIO.Write(sOut);
    }


    // Close the File

    if (iError == util.utilFileIO.NOERROR)
      pIO.Close();
  }
}

/*
 *  @version 1.1 03/30/2012
 *  @author  John Victorine
 */
